/**
 * @file Plugin for inserting text long page breaks.
 */
( function() {
  var pluginRequires = [ 'fakeobjects' ];
  if (Drupal.ckeditor_ver == 3) {
    pluginRequires = [ 'fakeobjects', 'htmldataprocessor' ];
  }

  CKEDITOR.plugins.add( 'textlongpagebreak',
  {
    requires  : pluginRequires,

    init : function( editor )
    {
      var addCssObj = CKEDITOR;

      if (Drupal.ckeditor_ver == 3) {
        addCssObj = editor;
      }
      // Add the styles that renders our fake objects.
      addCssObj.addCss(
        'img.text-long-pagebreak' +
        '{' +
        'background-image: url(' + CKEDITOR.getUrl( this.path + 'images/pagebreak.gif' ) + ');' +
        'background-position: center center;' +
        'background-repeat: no-repeat;' +
        'clear: both;' +
        'display: block;' +
        'float: none;' +
        'width: 100%;' +
        'border-top: #999999 1px dotted;' +
        'border-bottom: #999999 1px dotted;' +
        'height: 5px !important;' +
        '}'
        );

      // Register the toolbar button.
      editor.ui.addButton( 'TextLongPageBreak',
      {
        label : Drupal.t( 'Insert Page Break' ),
        icon : this.path + 'images/textlongpagebreak.png',
        command : 'textlongpagebreak'
      });

      editor.addCommand( 'textlongpagebreak',
      {
        exec : function()
        {
          insertComment( 'text-long-pagebreak' );
        }
      } );

      // This function effectively inserts the comment into the editor.
      function insertComment( text )
      {
        // Create the fake element that will be inserted into the document.
        // The trick is declaring it as an <hr>, so it will behave like a
        // block element (and in effect it behaves much like an <hr>).
        if ( !CKEDITOR.dom.comment.prototype.getAttribute ) {
          CKEDITOR.dom.comment.prototype.getAttribute = function() {
            return '';
          };
          CKEDITOR.dom.comment.prototype.attributes = {
            align : ''
          };
        }
        var fakeElement = editor.createFakeElement( new CKEDITOR.dom.comment( text ), text, 'hr' );

        // This is the trick part. We can't use editor.insertElement()
        // because we need to put the comment directly at <body> level.
        // We need to do range manipulation for that.

        // Get a DOM range from the current selection.
        var range = editor.getSelection().getRanges()[0],
        elementsPath = new CKEDITOR.dom.elementPath( range.getCommonAncestor( true ) ),
        element = ( elementsPath.block && elementsPath.block.getParent() ) || elementsPath.blockLimit,
        hasMoved;

        // If we're not in <body> go moving the position to after the
        // elements until reaching it. This may happen when inside tables,
        // lists, blockquotes, etc.
        while ( element && element.getName() != 'body' )
        {
          range.moveToPosition( element, CKEDITOR.POSITION_AFTER_END );
          hasMoved = 1;
          element = element.getParent();
        }

        // Split the current block.
        if ( !hasMoved )
          range.splitBlock( 'p' );

        // Insert the fake element into the document.
        range.insertNode( fakeElement );

        // Now, we move the selection to the best possible place following
        // our fake element.
        var next = fakeElement;
        while ( ( next = next.getNext() ) && !range.moveToElementEditStart( next ) )
        {}

        range.select();
      }
    },

    afterInit : function( editor )
    {
      // Adds the comment processing rules to the data filter, so comments
      // are replaced by fake elements.
      editor.dataProcessor.dataFilter.addRules(
      {
        comment : function( value )
        {
          if ( !CKEDITOR.htmlParser.comment.prototype.getAttribute ) {
            CKEDITOR.htmlParser.comment.prototype.getAttribute = function() {
              return '';
            };
            CKEDITOR.htmlParser.comment.prototype.attributes = {
              align : ''
            };
          }

          if ( value === 'text-long-pagebreak' ) {
            return editor.createFakeParserElement(new CKEDITOR.htmlParser.comment(value), 'text-long-pagebreak', 'hr');
          }

          return value;
        }
      });
    }
  });
} )();
